@@ -4,7 +4,6 @@ from simple_ddl_parser.parser import Parser
 from simple_ddl_parser.dialects.hql import HQL
 
 _ref = "REFERENCES"
-_def = "DEFAULT"
 _cons = "CONSTRAINT"
 
 
@@ -33,6 +32,8 @@ class DDLParser(Parser, HQL):
 
     columns_defenition = {
         "ON": "ON",
+        "DELETE": "DELETE",
+        "UPDATE": "UPDATE",
         "NULL": "NULL",
         "PRIMARY": "PRIMARY",
         "DEFAULT": "DEFAULT",
@@ -63,7 +64,7 @@ class DDLParser(Parser, HQL):
         "CACHE": "CACHE",
     }
     tokens = tuple(
-        ["ID", "NEWLINE", "DOT", "STRING", "LP", "RP"]
+        ["ID", "DOT", "STRING", "LP", "RP", "LT", "RT", "COMMAT"]
         + list(defenition_statements.values())
         + list(columns_defenition.values())
         + list(sequence_reserved.values())
@@ -77,7 +78,10 @@ class DDLParser(Parser, HQL):
     last_token = False
     columns_def = False
     after_columns = False
-
+    check = False
+    is_table = False
+    lt_open = 0
+    
     def t_STRING(self, t):
         r"\'[a-zA-Z_,0-9:><\=\-\+\~\%$'\!(){}\[\]\/\\\"]*\'\B"
         t.type = "STRING"
@@ -91,10 +95,18 @@ class DDLParser(Parser, HQL):
             t.type = "LP"
             if not self.after_columns:
                 self.columns_def = True
+        # todo: need to find less hacky way to parse HQL structure types
+        elif '<' == t.value and not self.check:
+            t.type = "LT"
+            self.lt_open += 1
+        elif '>' == t.value and not self.check:
+            t.type = "RT"
+            self.lt_open -= 1
         else:
             t.type = self.defenition_statements.get(
                 t.value.upper(), "ID"
             )  # Check for reserved word
+        
         if t.value.strip() == "'":
             self.string = True
         if t.type == "CREATE":
@@ -114,10 +126,14 @@ class DDLParser(Parser, HQL):
             t.type = "ARRAY"
         if t.type == "TABLE" or t.type == "INDEX":
             self.is_table = True
-        elif t.type == "SEQUENCE" and self.__dict__.get("is_table"):
+        elif t.type == "SEQUENCE" and self.is_table:
             t.type = "ID"
         if t.type == "SEQUENCE":
             self.sequence = True
+        if t.type == 'COMMA' and self.lt_open:
+            t.type = 'COMMAT'
+        if t.type == "CHECK":
+            self.check = True
         if t.type != "ID":
             t.value = t.value.upper()
         self.last_token = t.type
@@ -291,13 +307,30 @@ class DDLParser(Parser, HQL):
         if p[2].upper() == "EXTERNAL":
             external = True
         p[0] = {"external": external}
-
+    
+    def p_tid(self, p):
+        """ tid : LT ID
+        | tid ID 
+        | tid COMMAT
+        | tid RT
+        """
+        if not isinstance(p[1], list):
+            p[0] = [p[1]]
+        else:
+            p[0] = p[1]
+        
+        for i in list(p)[2:]:
+            p[0][0] += i
+        
     def p_column(self, p):
         """column : ID ID
+        | ID tid
         | column LP ID RP
         | column ID
         | column LP ID COMMA ID RP
-        | column ARRAY
+        | column ARRAY 
+        | ID ARRAY tid 
+        | column tid 
 
         """
         if isinstance(p[1], dict):
@@ -313,6 +346,15 @@ class DDLParser(Parser, HQL):
             arr_split = p_list[-1].split("ARRAY")
             append = "[]" if not arr_split[-1] else arr_split[-1]
             p[0]["type"] = p[0]["type"] + append
+        elif isinstance(p_list[-1], list):
+            if len(p_list) == 4:
+                p[0]["type"] = f'{p[2]} {p[3][0]}'
+            elif p[0]["type"]:
+                if len(p[0]["type"]) == 1 and isinstance(p[0]["type"], list):
+                    p[0]["type"] = p[0]["type"][0]
+                p[0]["type"] = f'{p[0]["type"]} {p_list[-1][0]}'
+            else:
+                p[0]["type"] = p_list[-1][0]
         else:
             match = re.match(r"[0-9]+", p_list[2])
             if bool(match):
@@ -324,20 +366,28 @@ class DDLParser(Parser, HQL):
 
     def extract_references(self, p_list):
         ref_index = p_list.index(_ref)
+        ref = {
+            "table": None,
+            "columns": [None],
+            "schema": None,
+            "on_delete": None,
+            "on_update": None
+        }
         if not "." in p_list[ref_index:]:
-
-            references = {
-                "table": p_list[ref_index + 1],
-                "columns": p_list[-1],
-                "schema": None,
-            }
-        else:
-            references = {
+            ref.update({
+                    "table": p_list[ref_index + 1]})
+            if not len(p_list) == 3:
+                ref.update({
+                    "columns": p_list[-1]
+                })
+        else: 
+            ref.update({
                 "schema": p_list[ref_index + 1],
                 "columns": p_list[-1],
                 "table": p_list[ref_index + 3],
-            }
-        return references
+            })
+            
+        return ref
 
     def p_null(self, p):
         """null : NULL
@@ -392,9 +442,9 @@ class DDLParser(Parser, HQL):
         if ("KEY" in p or "key" in p) and ("PRIMARY" in p or "primary" in p):
             pk = True
             nullable = False
-        if "unique" in p or "UNIQUE" in p:
+        elif "unique" in p or "UNIQUE" in p:
             unique = True
-        if isinstance(p_list[-1], dict) and "references" in p_list[-1]:
+        elif isinstance(p_list[-1], dict) and "references" in p_list[-1]:
             p_list[-1]["references"]["column"] = p_list[-1]["references"]["columns"][0]
             del p_list[-1]["references"]["columns"]
             references = p_list[-1]["references"]
@@ -550,12 +600,27 @@ class DDLParser(Parser, HQL):
         p[0] = columns
 
     def p_ref(self, p):
-        """ref : REFERENCES ID LP pid RP
-        | REFERENCES ID DOT ID LP pid RP
+        """ref : REFERENCES ID DOT ID
+        | REFERENCES ID
+        | ref LP pid RP
+        | ref ON DELETE ID
+        | ref ON UPDATE ID
         """
         p_list = remove_par(list(p))
-        data = {"references": self.extract_references(p_list)}
-        p[0] = data
+        if isinstance(p[1], dict):
+            p[0] = p[1]
+            if 'ON' not in p_list:
+                p[0]["references"]['columns'] = p_list[-1]
+            else:
+                p[0]["references"]['columns'] = p[0]["references"].get('columns', [None])
+        else:
+            data = {"references": self.extract_references(p_list)}
+            p[0] = data
+        if 'ON' in p:
+            if 'DELETE' in p:
+                p[0]["references"]['on_delete'] = p_list[-1]
+            elif 'UPDATE' in p:
+                p[0]["references"]['on_update'] = p_list[-1]
 
     def p_expression_primary_key(self, p):
         "expr : pkey"
